-----------------------------------------------------------
-- foreign minister
-----------------------------------------------------------

require('ai_diplomacy')

function ForeignMinister_EvaluateDecision(agent, decision, scope) 
	-- default we approve any decision we can take, override in country specific ai if wanted
	-- also some random to spread out the decisions
	local score = math.mod( CCurrentGameState.GetAIRand(), 100)

	score = Utils.CallScoredCountryAI(agent:GetCountryTag(), 'ForeignMinister_EvaluateDecision', score, agent, decision, scope)

	if score < 25 then
		score = 0
	end
	return score
end

function ForeignMinister_Tick(minister)
	--local t = os.clock()

	-- run any decisions available
	minister:ExecuteDiploDecisions()

	if math.mod( CCurrentGameState.GetAIRand(), 7) == 0 then
		ForeignMinister_HandlePeace(minister)
	end
		
	if minister:GetCountry():IsAtWar() then
		if math.mod( CCurrentGameState.GetAIRand(), 7) == 0 then
			ForeignMinister_HandleWar(minister)
		end
	end
	
	--Utils.LUA_DEBUGOUT("ForeignMinister_Tick - " .. tostring(minister:GetCountryTag()) .. " - " .. os.clock() - t)
end

function ForeignMinister_HandleWar( minister )

	local ministerTag = minister:GetCountryTag()
	local ministerCountry = minister:GetCountry()
	local ai = minister:GetOwnerAI() 

	for neighborTag in ministerCountry:GetNeighbours() do

		local neigborCountry = neighborTag:GetCountry()
		for countryTag in ministerCountry:GetCurrentAtWarWith() do
		
			-- mutual neighbors would be nice to move through
			if neigborCountry:IsNeighbour( countryTag ) then
				local relation = ai:GetRelation(ministerTag, neighborTag)

			
				if not relation:HasMilitaryAccess() then
					local action = CMilitaryAccessAction(ministerTag, countryTag)

					if action:IsSelectable() then

						local acceptanceChance = action:GetAIAcceptance()
						local score = DiploScore_DemandMilitaryAccess( ai, ministerTag, countryTag, ministerTag )

						if score > 50 and acceptanceChance > 40 then
							minister:Propose( action, score )
						end
					end
				end
			end
		end -- war targets
	end -- neighbors

	
	-- check wars
	for diploStatus in ministerCountry:GetDiplomacy() do
		local target = diploStatus:GetTarget()
		if target:IsValid() and diploStatus:HasWar() then
			local war = diploStatus:GetWar()
			if war:IsLimited() then
				-- always call in puppets
				for vassalTag in ministerCountry:GetVassals() do	
					if not war:IsPartOfWar(vassalTag) then
						local action = CCallAllyAction( ministerTag, vassalTag, target )
						action:SetValue( true ) -- limited
						if action:IsSelectable() then
							ai:PostAction( action )
						end
					end
				end
				
				
				-- compare military power
				--local intel = CAIIntel(ministerTag, target)
				--local theirStrength = intel:CalculateTheirPercievedMilitaryStrengh()
				-- factor in their allies that are part of war
				--for hostileAllyTag in target:GetCountry():GetAllies() do
				--	if war:IsPartOfWar(hostileAllyTag) then
				--		local allyIntel = CAIIntel(ministerTag, target)
				---		local allyStrength = allyIntel:CalculateTheirPercievedMilitaryStrengh()
				--		theirStrength = theirStrength + allyStrength
				--	end
				--end
				
				-- do we want to call in help?
				if war:GetCurrentRunningTimeInMonths() > 5 then
					--local ourStrength = intel:CalculateOurMilitaryStrength()
					--local strengthFactor = ourStrength / theirStrength
					if ministerCountry:CalcDesperation():Get() > 0.4 then --strengthFactor < 1.4 then
						for allyTag in ministerCountry:GetAllies() do
							if not war:IsPartOfWar(allyTag) then
								local action = CCallAllyAction( ministerTag, allyTag, target )
								action:SetValue( true ) -- limited
								if action:IsSelectable() then
									ai:PostAction( action )
								end
							end
						end
					end
				end
			else -- not-limited, call in any faction members not there:
				for allyTag in ministerCountry:GetAllies() do
					if not war:IsPartOfWar(allyTag) then
						local action = CCallAllyAction( ministerTag, allyTag, target )
						action:SetValue( true ) -- limited
						if action:IsSelectable() then
							ai:PostAction( action )
						end
					end
				end
			end
		end
	end
end

function ForeignMinister_HandlePeace( minister )
	
	local ministerCountry = minister:GetCountry()
	local ministerTag = ministerCountry:GetCountryTag()
	local ai = minister:GetOwnerAI() 
	local strategy = ministerCountry:GetStrategy()
	
	local slider = ministerCountry:GetLeadershipDistributionAt( CDistributionSetting._LEADERSHIP_DIPLOMACY_ )
	local amount = slider:GetNeeded():Get()
	local influence = ministerCountry:GetTotalLeadership():Get() * 0.18	-- 18% of leadership should be influence..
	influence = influence * defines.economy.LEADERSHIP_TO_DIPLOMACY
	influence = influence - amount
	local numberOfInfluences = nil
	
	for country in CCurrentGameState.GetCountries() do
		local countryTag = country:GetCountryTag()

		if countryTag:IsValid() and country:Exists() and countryTag:IsReal() and
		   not (ministerCountry:HasDiplomatEnroute(countryTag)) and
		   not (countryTag == ministerTag) then

			local relation = ministerCountry:GetRelation( countryTag )

			-- strategy
			local strategy = ministerCountry:GetStrategy()
			local threat = relation:GetThreat():Get()
			local antagonism = strategy:GetAntagonism(countryTag)
			local friendliness = strategy:GetFriendliness(countryTag)
			local protectionism = strategy:GetProtectionism(countryTag)

			-- war
			-- non-scripted war declaration:
			--ProposeDeclareWar( minister, ai, ministerCountry, countryTag )

			-- alliance  -- we let player initiate this as it messes too much with history
			--if not relation:HasAlliance() and relation:GetValue():GetTruncated() > 0 then
			--	local action = CAllianceAction(ministerTag, countryTag)	
			--	if action:IsSelectable() then
			--		
			--		local score = DiploScore_Alliance( ai, ministerTag, countryTag, ministerTag )
			--		if score > 50 then
			--			local acceptanceChance = action:GetAIAcceptance()	
			--			if acceptanceChance > 40 then
			--				minister:Propose( action, score )
			--			end
			--		end
			--	end
			--end
			
			-- offer military access
			if not country:GetRelation(ministerTag):HasMilitaryAccess() then
				local action = COfferMilitaryAccessAction(ministerTag, countryTag)

				if action:IsSelectable() then
					local score = DiploScore_OfferMilitaryAccess( ai, ministerTag, countryTag, ministerTag )
					if score > 50 then
						local acceptanceChance = action:GetAIAcceptance()
						if acceptanceChance > 50 then
							minister:Propose( action, score )
						end
					end
				end
			end
			
			-- NAP-ing
			if not country:GetRelation(ministerTag):HasNap() then
				local action = CNapAction(ministerTag, countryTag)
				if action:IsSelectable() then
					--Utils.LUA_DEBUGOUT("NAP")
					local acceptanceChance = action:GetAIAcceptance()
					if acceptanceChance > 50 then
						local score = DiploScore_NonAgression( ai, ministerTag, countryTag, ministerTag )
						if score > 50 then
							minister:Propose( action, score )
						end
					end
				end
			end

			if not relation:IsGuaranting() then
				local action = CGuaranteeAction(ministerTag, countryTag)
				if action:IsSelectable() then
					local score = DiploScore_Guarantee( ai, ministerTag, countryTag, ministerTag )
					if score > 50 then
						minister:Propose( action, score )
					end
				end
			end
			
			-- invite/influence to faction
			if ministerCountry:HasFaction() then

				if not country:HasFaction() then

					local action = CFactionAction(ministerTag, countryTag)
					action:SetValue(false)

					if action:IsSelectable() then
						local acceptanceChance = action:GetAIAcceptance()
						if acceptanceChance > 40 then
							local score = DiploScore_InviteToFaction( ai, ministerTag, countryTag, ministerTag )
							if score > 50 then
								minister:Propose( action, score )
							end
						end
					end

					local influenceAction = CInfluenceNation(ministerTag, countryTag)
					if ai:IsInfluencing(ministerTag, countryTag) then
						influenceAction:SetValue(false)
						if influenceAction:IsSelectable() then
							local score = DiploScore_InfluenceNation( ai, ministerTag, countryTag, ministerTag )
							if score < 40 then
								--Utils.LUA_DEBUGOUT(" CANCELED INFLUENCE : " .. tostring(ministerTag) .. " -> " .. tostring(countryTag))
								minister:Propose( influenceAction, 1000 )
							end								
						end
					else
						if influenceAction:IsSelectable() then
							if numberOfInfluences == nil then
								numberOfInfluences = ministerCountry:CalculateNumberOfActiveInfluences()
							end
							if influence > 0 and numberOfInfluences < 3 then
								local score = DiploScore_InfluenceNation( ai, ministerTag, countryTag, ministerTag )
								if score > 50 then
									--Utils.LUA_DEBUGOUT("INFLUENCE : " .. tostring(ministerTag) .. " -> " .. tostring(countryTag))
									--minister:Propose( influenceAction, score )
									ai:PostAction( influenceAction )
									numberOfInfluences = numberOfInfluences + 1
								end
							end
						end
					end
				end
			end
						
			-- see if we can help out
			if not country:GetRelation(ministerTag):AllowDebts() then
				
				local action = CDebtAction(ministerTag, countryTag)
				if ministerCountry:GetTotalIC() < country:GetTotalIC() then
					if action:IsSelectable() then
						local score = DiploScore_Debt( ai, ministerTag, countryTag, ministerTag )
						if score > 50 then
							local acceptanceChance = action:GetAIAcceptance()
							if acceptanceChance > 50 then
								minister:Propose( action, score )
							end
						end
					end
				end
			end
			
			-- embargos
			if relation:HasEmbargo() then
				-- do we want to stop embargoing?
				local action = CEmbargoAction(ministerTag, countryTag)
				action:SetValue(false)
				
				if action:IsSelectable() then
					local embargoScore = DiploScore_Embargo( ai, ministerTag, countryTag, ministerTag )

					if embargoScore < 40 then
						minister:Propose( action, 100 )
					end
				end
				
			else
				local action = CEmbargoAction(ministerTag, countryTag)

				if action:IsSelectable() then
					local score = DiploScore_Embargo( ai, ministerTag, countryTag, ministerTag )

					if score > 50 then
						minister:Propose( action, score )
					end
				end
			
			end
			
			--if warDesirability < 0 and ai:IsPreparingWarWith( countryTag ) then
			--	-- no longer a good idea
			--	ai:CancelPrepareWar( countryTag )
			--	if minister:GetProposedWarTarget() == countryTag then
			--		ClearWarProposal()
			--	end
			--end

		end -- country:Exists()
	end -- countries
	
	-- scripted wars
	Utils.CallCountryAI( ministerTag, 'ProposeDeclareWar', minister )
	
	-- are we interested in joining a faction?
	if (not ministerCountry:HasFaction()) and ministerCountry:GetEffectiveNeutrality():Get() < 65 then
		local bestFaction = nil
		local bestScore = -1
		for faction in CCurrentGameState.GetFactions() do

			if faction:IsValid() then

				-- evaluate faction
				local progress = faction:GetNormalizedProgress()
				local score = progress:Get()
				local dist = minister:GetOwnerAI():GetNormalizedAlignmentDistance(ministerCountry, faction):Get()
				if dist > 0.0 and dist < 4.0 then
					local factor = 1.0 - ( (dist - 1.0)/ (4.0 - 1) )
					score = score + 100 * factor
				end

				Utils.CallScoredCountryAI(ministerTag, 'DiploScore_JoinFaction', score, minister, faction)

				if bestScore < score then
					bestScore = score
					bestFaction = faction
				end
			end
		end

		if bestFaction and bestScore > 50 then
			local action = CInfluenceAllianceLeader( ministerTag, bestFaction:GetFactionLeader() )
			if action:IsSelectable() then
				minister:Propose( action, bestScore )
			end
		end
	end

	-- break any deals?
	for alliedCountry in ministerCountry:GetAllies() do
		local rel = ministerCountry:GetRelation( alliedCountry )
		local relValue = rel:GetValue():GetTruncated()

		local threat = rel:GetThreat():Get()
		threat = threat * CalculateAlignmentFactor(ai, ministerCountry, alliedCountry:GetCountry())
		local antagonism = strategy:GetAntagonism( alliedCountry ) / 4
		local friendliness = strategy:GetFriendliness( alliedCountry ) / 4

		local score = ((antagonism + threat) / 2.0) - friendliness
		score = score - relValue / 2.0

		if score >= 100 then
			local action = CAllianceAction(ministerTag, alliedCountry)
			action:SetValue(false) -- cancel
			if action:IsSelectable() then
				minister:Propose( action, score )
			end
		end
	end
		
	--EvalutateExistingTrades(minister)

end

function ProposeDeclareWar( minister, ai, ministerCountry, countryTag )

	local warDesirability = CalculateWarDesirability( ai, ministerCountry, countryTag )

	if warDesirability > 40 then
		minister:ProposeWar( countryTag, warDesirability )
	end


	-- check if we would like to do a limited war instead (axis only)
	if ministerCountry:GetRules():GetValue( CRule._RULE_LIMITED_WAR_ ) then
		local limitedWarScore = EvaluateLimitedWar( minister, countryTag, warDesirability )
		if limitedWarScore > 50 then
			local action = CDeclareWarAction(ministerTag, countryTag)
			action:SetValue(false)
			minister:Propose( action, limitedWarScore )
		end
	end
end
	
	
-- Check if we want to do a limited war 
function EvaluateLimitedWar(minister, target, warDesirability )
    local score = 0
    if warDesirability > 80 then 
		score = 100
    end
    return Utils.CallScoredCountryAI(minister:GetCountryTag(), 'EvaluateLimitedWar', 
									 score, minister, target, warDesirability)
end


function ProposeTrades(ai, ministerTag)
	local SMALLEST_TRADE = 0.03
	local ministerCountry = ministerTag:GetCountry()
	local strategy = ministerCountry:GetStrategy()
	
	local bestScore = -10000
	local bestAction = nil

	-- see what we are low on and find someone who is hoarding it
	local MAX_GOODS = CGoodsPool._GC_NUMOF_-1
	local myMoney = ministerCountry:GetMoneyBalanceAverage():Get()
	local myMoneyPool = ministerCountry:GetPool():Get( CGoodsPool._MONEY_ ):Get()
	
	if myMoneyPool > 30 and myMoney < 0.01 then
		myMoney = myMoney + myMoneyPool * 0.05
	end
	
	for goods = 0, MAX_GOODS do
		if (not (goods == CGoodsPool._MONEY_)) and (not (goods == CGoodsPool._FUEL_)) then
			local balance = ministerCountry:GetDailyBalance( goods ):Get()
			if balance < 0.01 and myMoney > 0.001 then

				for country in CCurrentGameState.GetCountries() do
					local countryTag = country:GetCountryTag()

					if country:Exists() and countryTag:IsReal() and
					not (ministerCountry:HasDiplomatEnroute(countryTag)) and
					not (countryTag == ministerTag) then
					
						local rel = ministerCountry:GetRelation( countryTag )
				
						local theirBalance = country:GetDailyBalance(goods):Get() * 0.6
						if goods == CGoodsPool._SUPPLIES_ then
							theirBalance = math.min( theirBalance, country:GetSupplyBalanceAverage():Get() * 0.6 )
						end
						
						if theirBalance > 0.01 then
							local requested = math.min( theirBalance, -balance )
							local action = CTradeAction( ministerTag, countryTag )

							if action:IsValid() and action:IsSelectable() then
								action:SetTrading( CFixedPoint(requested), goods )
								local money = action:GetTrading( CGoodsPool._MONEY_, ministerTag ):Get()
								
								local factor = 1.0
								if money > myMoney then
									factor = myMoney / money - 0.08
									
									if factor > 0.01 then
										local action2 = CTradeAction( ministerTag, countryTag )
										local amount = requested * factor
										action2:SetTrading( CFixedPoint(amount), goods )
										local money2 = action2:GetTrading( CGoodsPool._MONEY_, ministerTag ):Get()
										if amount > SMALLEST_TRADE and money2 > SMALLEST_TRADE then -- remove small stuff
											local score = 70 --balance
											score = score - rel:GetThreat():Get() * CalculateAlignmentFactor(ai, ministerCountry, country)
											score = score + (100 * amount / math.abs(balance))
											score = score + math.floor(theirBalance / 10) -- big producer bonus = more stable
											
											-- we need transports to trade
											if ministerCountry:NeedConvoyToTradeWith( countryTag ) then
												if action2:GetRoute():GetConvoyResponsible() == ministerTag then
													if ministerCountry:GetTransports() == 0 then
														score = 0
													end
												else
													if countryTag:GetCountry():IsAtWar() then
														score = score - 20
													end
												end
											end
											
											local acceptanceChance = action2:GetAIAcceptance()
											acceptanceChance = acceptanceChance - ai:GetSpamPenalty(countryTag)
											if ( factor > 0.01 and acceptanceChance > 50 ) then
												if score > bestScore and action2:IsConvoyPossible() 
												and (not ai:AlreadyTradingResourceOtherWay( action2:GetRoute() ) )
												and (not ai:AlreadyTradingDisabledResource( action2:GetRoute() ))
												and action2:IsSelectable()
												then
													bestAction = action2
													bestScore = score
												end
											end
										end -- if amount
									end
								elseif requested > SMALLEST_TRADE and money > SMALLEST_TRADE then
									local score = 70 --balance
									
									score = score - rel:GetThreat():Get() * CalculateAlignmentFactor(ai, ministerCountry, country)
									score = score + (100 * requested / math.abs(balance))
									score = score + math.floor(theirBalance / 10) -- big producer bonus = more stable
									
									-- we need transports to trade
									if ministerCountry:NeedConvoyToTradeWith( countryTag ) then
										if action:GetRoute():GetConvoyResponsible() == ministerTag then
											if ministerCountry:GetTransports() == 0 then
												score = 0
											end
										else
											if countryTag:GetCountry():IsAtWar() then
												score = score - 20
											end
										end
									end
										
									local acceptanceChance = action:GetAIAcceptance()
									acceptanceChance = acceptanceChance - ai:GetSpamPenalty(countryTag)
									if ( factor > 0.01 and acceptanceChance > 50 ) then
										if score > bestScore and action:IsConvoyPossible()
										and (not ai:AlreadyTradingResourceOtherWay( action:GetRoute() ) )
										and (not ai:AlreadyTradingDisabledResource( action:GetRoute() ))
										then
											bestAction = action
											bestScore = score
										end
									end
								end
							end
						end
					end	
				end
			end
		end	
	end
	
	if bestAction then
		ai:PostAction( bestAction )
	end
	
end


function ForeignMinister_OnWar( agent, countryTag1, countryTag2, war )
	if war:IsLimited() then
		-- dont pull anything else right now, lets wait until we need it
	end
end

	
function EvalutateExistingTrades(ai, ministerTag)
	--Utils.LUA_DEBUGOUT( "EvalutateExistingTrades - " .. tostring(ministerTag) )	
	local ministerCountry = ministerTag:GetCountry()
	for country in CCurrentGameState.GetCountries() do
		if (not (country:GetCountryTag() == ministerTag)) then
			local routes = ministerCountry:GetRelation( country:GetCountryTag() ):GetTradeRoutes()
			for route in routes do
				--if route:IsValid() then
					if route:IsInactive() and ai:HasTradeGoneStale(route) then
						local action = CTradeAction(ministerTag, country:GetCountryTag())
						action:SetRoute(route)
						action:SetValue(false)
						if action:IsSelectable() then
							ai:PostAction(action)
						end
					-- trading resource we are deficient in?
					elseif ai:IsTradeingAwayNeededResource(route) then
						local action = CTradeAction(ministerTag, country:GetCountryTag())
						action:SetRoute(route)
						action:SetValue(false)
						--Utils.LUA_DEBUGOUT("CANCELING TRADE (" .. tostring(ministerTag) .. "): " .. tostring(country:GetCountryTag()) )
						if action:IsSelectable() then
							ai:PostAction(action)
						end
					end
				--end
			end
		end
	end
	
end

function ForeignMinister_ManageTrade(ai, ministerTag)
	--Utils.LUA_DEBUGOUT("ForeignMinister_ManageTrade - " .. tostring(ministerTag))
	
	if math.mod( CCurrentGameState.GetAIRand(), 6) == 0 then
		ProposeTrades(ai, ministerTag)
		EvalutateExistingTrades(ai, ministerTag)
	end
end
