-----------------------------------------------------------
-- production
-----------------------------------------------------------

function ProductionMinister_Tick(minister)
	--BalanceProductionSliders( minister:GetOwnerAI(), minister:GetCountry() )
	
	ManageProduction(minister)
end

local PRIO_SETTINGS = {
	[0] = { -- full ai automation
		[0] = CDistributionSetting._PRODUCTION_CONSUMER_, 
		CDistributionSetting._PRODUCTION_SUPPLY_,
		CDistributionSetting._PRODUCTION_REINFORCEMENT_,	
		CDistributionSetting._PRODUCTION_PRODUCTION_,
		CDistributionSetting._PRODUCTION_UPGRADE_
	},
	[1] = {  -- prio production
		[0] = CDistributionSetting._PRODUCTION_CONSUMER_, 
		CDistributionSetting._PRODUCTION_SUPPLY_,
		CDistributionSetting._PRODUCTION_PRODUCTION_,
		CDistributionSetting._PRODUCTION_REINFORCEMENT_,
		CDistributionSetting._PRODUCTION_UPGRADE_
	},
	[2] = {   -- prio upgrades
		[0] = CDistributionSetting._PRODUCTION_CONSUMER_, 
		CDistributionSetting._PRODUCTION_SUPPLY_,
		CDistributionSetting._PRODUCTION_UPGRADE_,
		CDistributionSetting._PRODUCTION_REINFORCEMENT_,	
		CDistributionSetting._PRODUCTION_PRODUCTION_
	},
	[3] = {   -- prio reinforcement
		[0] = CDistributionSetting._PRODUCTION_CONSUMER_, 
		CDistributionSetting._PRODUCTION_SUPPLY_,
		CDistributionSetting._PRODUCTION_REINFORCEMENT_,	
		CDistributionSetting._PRODUCTION_PRODUCTION_,
		CDistributionSetting._PRODUCTION_UPGRADE_
	},
	[4] = {   -- prio special reinforcement case
		[0] = CDistributionSetting._PRODUCTION_REINFORCEMENT_,	
		CDistributionSetting._PRODUCTION_CONSUMER_, 
		CDistributionSetting._PRODUCTION_SUPPLY_,
		CDistributionSetting._PRODUCTION_PRODUCTION_,
		CDistributionSetting._PRODUCTION_UPGRADE_
	}
}

function BalanceProductionSliders(ai, ministerCountry, prioSelection)
	--Utils.LUA_DEBUGOUT("BalanceProductionSliders - " .. tostring(ministerCountry:GetCountryTag()) )
		
	-- If country just started mobilizing (or gets bonus reinforcements for some other reason), boost reinforcements
	if ( prioSelection == 0 or prioSelection == 3 )then
		local reinforcement_multiplier = ministerCountry:CalculateReinforcementMultiplier():Get()
		if reinforcement_multiplier > 1.0 then
			prioSelection = 4
		end
	end
	
	local sum = 0
	local changes = CArrayFix(5)	
	
	local totalIC = ministerCountry:GetTotalIC()
	for setting = 0, table.getn( PRIO_SETTINGS[prioSelection] ) do
		local slider = ministerCountry:GetProductionDistributionAt( setting )
		local percentage = slider:GetNeeded():Get() / totalIC
		percentage = percentage * 100
		changes:SetAt( setting, CFixedPoint( math.max(percentage, 0.0 ) ) )
	end
	
	-- special considerations
	local dissent = ministerCountry:GetDissent():Get()
	if dissent > 0.01 then -- fight dissent 
		changes:SetAt( CDistributionSetting._PRODUCTION_CONSUMER_, changes:GetAt( CDistributionSetting._PRODUCTION_CONSUMER_ ) + 10 )
	end
	
	local supplyStockpile = ministerCountry:GetPool():Get( CGoodsPool._SUPPLIES_ ):Get()
	if prioSelection == 0 then
		local weeksSupplyUse = ministerCountry:GetDailyExpense( CGoodsPool._SUPPLIES_ ):Get() * 7
		if (supplyStockpile < weeksSupplyUse * 8.0) then
			changes:SetAt( CDistributionSetting._PRODUCTION_SUPPLY_, changes:GetAt( CDistributionSetting._PRODUCTION_SUPPLY_ ) + 10 )
		elseif (supplyStockpile > weeksSupplyUse * 20.0) and supplyStockpile > 4000 then
			local newVal = changes:GetAt( CDistributionSetting._PRODUCTION_SUPPLY_ ):Get() - 10
			if newVal < 0 then
				newVal = 0.0
			end
			changes:SetAt( CDistributionSetting._PRODUCTION_SUPPLY_, CFixedPoint(newVal) )
		end
	end
	
	if supplyStockpile > 50000 then -- close to cap
		local newVal = changes:GetAt( CDistributionSetting._PRODUCTION_SUPPLY_ ):Get() - 10
		if newVal < 0 then
			newVal = 0.0
		end
		changes:SetAt( CDistributionSetting._PRODUCTION_SUPPLY_, CFixedPoint(newVal) )
	end
	
	
	local factor_left = 100
	for i = 0, table.getn( PRIO_SETTINGS[prioSelection] ) do -- normalize
		local index = PRIO_SETTINGS[prioSelection][i]
		local factor = changes:GetAt( index ):Get() + 0.005 -- stop dumb roundoff errors
		factor = math.min(factor, factor_left)
		factor_left = math.max(factor_left - factor, 0.0)

		changes:SetAt( index, CFixedPoint(factor) )
	end
	
	factor_left = math.max(factor_left, 0.0)
	if prioSelection == 0 then
	                
		local neededUpgrades = ministerCountry:GetProductionDistributionAt( CDistributionSetting._PRODUCTION_UPGRADE_ ):GetNeeded():Get()
		local neededPercentage = ((neededUpgrades * 0.1) / totalIC) * 100
		local currentPercentage =  changes:GetAt(CDistributionSetting._PRODUCTION_UPGRADE_):Get()
		if currentPercentage < neededPercentage then
			local prodAdd = math.min(factor_left, neededPercentage - currentPercentage)
			factor_left = factor_left - prodAdd
			changes:SetAt( CDistributionSetting._PRODUCTION_UPGRADE_, 
		               		changes:GetAt(CDistributionSetting._PRODUCTION_UPGRADE_) + prodAdd )
		end
	
		changes:SetAt( CDistributionSetting._PRODUCTION_PRODUCTION_, 
		               changes:GetAt(CDistributionSetting._PRODUCTION_PRODUCTION_) + factor_left )
	else
		if dissent > 0.01
		then
			changes:SetAt( CDistributionSetting._PRODUCTION_CONSUMER_, 
			               changes:GetAt( CDistributionSetting._PRODUCTION_CONSUMER_ ) + factor_left )
		else
			changes:SetAt( CDistributionSetting._PRODUCTION_SUPPLY_, 
			               changes:GetAt(CDistributionSetting._PRODUCTION_SUPPLY_) + factor_left )	
		end
	end
	
	local command = CChangeInvestmentCommand( ministerCountry:GetCountryTag(), changes )
	ai:Post( command )
end


-- for debugging
--local GOODS_TO_STRING = { [0] = "_SUPPLIES_","_FUEL_",	"_MONEY_",	"_CRUDE_OIL_",	"_METAL_",	"_ENERGY_",	"_RARE_MATERIALS_" }


function ManageProduction(minister)
	local ministerTag = minister:GetCountryTag()
	local ministerCountry = minister:GetCountry()
	local ai = minister:GetOwnerAI() 
	local strategy = ministerCountry:GetStrategy()
	local capitalProvId =  ministerCountry:GetActingCapitalLocation():GetProvinceID()
	
	local ic = ministerCountry:GetICPart( CDistributionSetting._PRODUCTION_PRODUCTION_ ):Get()
	ic = ic - ministerCountry:GetUsedIC():Get()
	
	-- we need convoys at all?
	ic = ConstructConvoys(ai, minister, ministerTag, ministerCountry, ic )
	
	-- ai list of requests, in prio order
	if  ministerCountry:IsAtWar() or ministerCountry:HasExtraManpowerLeft() then
		local bBuildReserveAtPeace = not ministerCountry:IsAtWar()
		local requestQueue = ai:GetReqProdQueue()
		--while (not requestQueue:IsEmpty()) and ic >= 0.0 do
		for request in ai:GetReqProdQueueIter() do
			local unit = request.pUnit --requestQueue:GetTailData().pUnit
			local bBuildReserve = bBuildReserveAtPeace and unit:IsRegiment()
			local tmp = ministerCountry:GetBuildCostIC( unit, 1, bBuildReserve ):Get()
				
			local res = ic / tmp
			if res > 0.2 then
				ic = ic - tmp
				ai:GetReqProdQueue():Remove(request)

				local orderlist = SubUnitList()
				SubUnitList.Append( orderlist, unit )

				local construct = CConstructUnitCommand( ministerTag, orderlist, capitalProvId, 1, bBuildReserve, CNullTag(), CID() )
				ai:Post( construct )
			end
		end
	end
	
	-- Buildings
	if ic >= 0 then
		if not ( ministerCountry:IsAtWar() or ministerCountry:GetStrategy():IsPreparingWar() ) then 
			local factory = CBuildingDataBase.GetBuilding( "industry" )
			local factoryCost = ministerCountry:GetBuildCost( factory ):Get()
			
			for provinceId in ministerCountry:GetOwnedProvinces() do
				local province = CCurrentGameState.GetProvince( provinceId )
				
				--Utils.LUA_DEBUGOUT("b4")
				ic = ic - factoryCost
				if ic < 0.0 then		
					break
				end
				--Utils.LUA_DEBUGOUT("build with " .. tostring(ministerTag) )
								
				if ministerCountry:IsBuildingAllowed(factory, province)
				and (not province:IsFrontProvince(false) )
				--and (not province:HasAdjacentEnemyOrCB() ) 
				and province:GetInfrastructure():Get() > 0.3 then
					local constructCommand = CConstructBuildingCommand( ministerTag, factory, provinceId, 1 )
					if constructCommand:IsValid() then
						ai:Post( constructCommand )
						--Utils.LUA_DEBUGOUT("building factory in " .. provinceId .. 
						--				  " for " .. tostring(ministerTag) .. 
						--				  " costing: " .. factoryCost ..
						--				  " of IC left: " .. ic )
						break -- one enough for now
					end
				end
			end
		end
		
		-- look at building defences
		local costalFort = CBuildingDataBase.GetBuilding( "coastal_fort" )
		local costalFortCost = ministerCountry:GetBuildCost( costalFort ):Get()
		--if ministerCountry:GetStrategicWarfare():GetBombingImpact():Get() > 0.3 then 
		for navalBaseProvince in ministerCountry:GetNavalBases() do
			--Utils.LUA_DEBUGOUT("navalBaseProvince " )
			if not navalBaseProvince:HasBuilding( costalFort ) then
				--Utils.LUA_DEBUGOUT("WANT building coastal fort in " .. navalBaseProvince:GetProvinceID() .. 
				--					  " for " .. tostring(ministerTag) .. 
				--					  " costing: " .. costalFortCost ..
				--					  " of IC left: " .. ic )
				if ministerCountry:IsBuildingAllowed(costalFort, navalBaseProvince) then
					--Utils.LUA_DEBUGOUT("building is allowed")
					ic = ic - costalFortCost
					if ic < 0.0 then		
						break
					end
					--Utils.LUA_DEBUGOUT("building coastal fort in " .. navalBaseProvince:GetProvinceID() .. 
					--				  " for " .. tostring(ministerTag) .. 
					--				  " costing: " .. costalFortCost ..
					--				  " of IC left: " .. ic )
				  
					local constructCommand = CConstructBuildingCommand( ministerTag, costalFort, navalBaseProvince:GetProvinceID(), 1 )
					if constructCommand:IsValid() then
						ai:Post( constructCommand )

						break -- one enough for now
					end
				end
			end
		end
		
		local antiAir = CBuildingDataBase.GetBuilding( "anti_air" )
		local antiAirCost = ministerCountry:GetBuildCost( antiAir):Get()
		for airBaseProvince in ministerCountry:GetAirBases() do
			--Utils.LUA_DEBUGOUT("airBaseProvince " )
			if not airBaseProvince:HasBuilding( antiAir ) then
				--Utils.LUA_DEBUGOUT("WANT building AA in " .. airBaseProvince:GetProvinceID() .. 
				--					  " for " .. tostring(ministerTag) .. 
				--					  " costing: " .. antiAirCost ..
				--					  " of IC left: " .. ic )
				if ministerCountry:IsBuildingAllowed(antiAir, airBaseProvince) then
					--Utils.LUA_DEBUGOUT("building is allowed")
					ic = ic - antiAirCost
					if ic < 0.0 then		
						break
					end
					--Utils.LUA_DEBUGOUT("building AA in " .. airBaseProvince:GetProvinceID() .. 
					---				  " for " .. tostring(ministerTag) .. 
					--				  " costing: " .. antiAirCost ..
					--				  " of IC left: " .. ic )
				  
					local constructCommand = CConstructBuildingCommand( ministerTag, antiAir, airBaseProvince:GetProvinceID(), 1 )
					if constructCommand:IsValid() then
						ai:Post( constructCommand )

						break -- one enough for now
					end
				end
			end
		end
	end
	
	if math.mod( CCurrentGameState.GetAIRand(), 7) == 0 then
		minister:PrioritizeBuildQueue()
	end
	
	--Utils.LUA_DEBUGOUT( "ic at end > " .. tostring(ic) )
	--local manpower = ministerCountry:GetManpower():Get()
	--Utils.LUA_DEBUGOUT( "mp at end > " .. tostring(manpower) )

	-- always fill out with more infantry if possible.
	--if ( ic > 0 and manpower > 10) then
		--Utils.LUA_DEBUGOUT( "has ic and manpower" )

	--	local orderlist = SubUnitList()
	--	local infantry = CSubUnitDataBase.GetSubUnit("infantry_brigade")
	--	local militia = CSubUnitDataBase.GetSubUnit("militia_brigade")
	--	if ministerCountry:GetTechnologyStatus():IsUnitAvailable(infantry) then 
	--		SubUnitList.Append( orderlist, infantry )
	--		SubUnitList.Append( orderlist, infantry )
	--		SubUnitList.Append( orderlist, infantry )
	--	else
	--		SubUnitList.Append( orderlist, militia )
	--		SubUnitList.Append( orderlist, militia )
	--		SubUnitList.Append( orderlist, militia )
	--	end
		--Utils.LUA_DEBUGOUT( "builds a division." )
	--	local construct = CConstructUnitCommand( ministerTag, orderlist, capitalProvId, 1, bBuildReserveAtPeace, CNullTag(), CID() )
	--	ai:Post( construct )
	--end
end


function ConstructConvoys(ai, minister, ministerTag, ministerCountry, ic )

	local ministerPortCount = ministerCountry:GetNumOfPorts()
	if ministerPortCount > 0 then
		local freeTransports = ministerCountry:GetTransports() 
		if freeTransports < ministerPortCount*2 then
			local neededTransports = ministerCountry:GetTotalNeededTransports()
			neededTransports = neededTransports + (ministerPortCount - freeTransports) -- want some spares
			neededTransports = neededTransports - minister:CountTransportsUnderConstruction()
			if neededTransports > 0 then
				local buildRequestCount = neededTransports / defines.economy.CONVOY_CONSTRUCTION_SIZE;
				buildRequestCount = math.ceil( math.max( buildRequestCount, 1) )
				
				local cost = ministerCountry:GetConvoyBuildCost():Get()
				local res = ic / cost
				if true then --res > 0.3 then
					--buildRequestCount = math.min( buildRequestCount, math.floor(res + 0.5) )
					ic = ic - buildRequestCount * cost
					local transportCommand = CConstructConvoyCommand( ministerTag, false, 1)--buildRequestCount )
					ai:Post( transportCommand )
					--Utils.LUA_DEBUGOUT("build convoys for " .. tostring(ministerTag) )
					--Utils.LUA_DEBUGOUT("build convoys: " .. tostring(buildRequestCount) )
				end
			end
		end
		
		-- if at war, we could use protection
		if minister:GetCountry():IsAtWar() then
			local neededEscorts = minister:CountTotalDesiredEscorts()
			neededEscorts = neededEscorts - minister:CountEscortsUnderConstruction()
			if neededEscorts > 0 then
				local buildRequestCount = neededEscorts / defines.economy.CONVOY_CONSTRUCTION_SIZE;
				buildRequestCount = math.ceil( math.max( buildRequestCount, 1) )
				
				local cost = ministerCountry:GetEscortBuildCost():Get()
				local res = ic / cost
				if res > 0.8 then
					buildRequestCount = math.min( buildRequestCount, math.floor(res + 0.5) )
					ic = ic - buildRequestCount * cost
					local escortCommand = CConstructConvoyCommand( ministerTag, true, buildRequestCount )
					ai:Post( escortCommand )
					--Utils.LUA_DEBUGOUT("build escorts for " .. tostring(ministerTag) )
					--Utils.LUA_DEBUGOUT("build escorts: " .. tostring(buildRequestCount) )
				end
			end
		end
		
	end
	return ic
end
