-----------------------------------------------------------
-- manage research
-----------------------------------------------------------

function BalanceLeadershipSliders(ai, ministerCountry)
	local PRIO_SETTINGS = {
		[0] = CDistributionSetting._LEADERSHIP_DIPLOMACY_,
			CDistributionSetting._LEADERSHIP_NCO_, 
			CDistributionSetting._LEADERSHIP_ESPIONAGE_, 
			CDistributionSetting._LEADERSHIP_RESEARCH_
	}
	
	local sum = 0
	local changes = CArrayFix(4)	


	-- Officer ratio.
	if ministerCountry:GetOfficerRatio():Get() < 0.75 then
		changes:SetAt( CDistributionSetting._LEADERSHIP_NCO_, CFixedPoint( 40 ) )
	elseif ministerCountry:GetOfficerRatio():Get() < 0.9 then
		changes:SetAt( CDistributionSetting._LEADERSHIP_NCO_, CFixedPoint( 30 ) )
	elseif ministerCountry:GetOfficerRatio():Get() < 1.2 then
		changes:SetAt( CDistributionSetting._LEADERSHIP_NCO_, CFixedPoint( 20 ) )
	else
		changes:SetAt( CDistributionSetting._LEADERSHIP_NCO_, CFixedPoint( 0 ) )
	end
	
	-- if we need DI, we take it.
	if ministerCountry:GetDiplomaticInfluence():Get() < 10 then
		changes:SetAt( CDistributionSetting._LEADERSHIP_DIPLOMACY_, CFixedPoint( 20 ) )
	else
		changes:SetAt( CDistributionSetting._LEADERSHIP_DIPLOMACY_, CFixedPoint( 0 ) )
	end
	
	
	
	-- always go for 10%, more is useless, unless if GiE
	if ministerCountry:IsGovernmentInExile() then
		changes:SetAt( CDistributionSetting._LEADERSHIP_ESPIONAGE_, CFixedPoint( 100 ) )
	else
		changes:SetAt( CDistributionSetting._LEADERSHIP_ESPIONAGE_, CFixedPoint(10 ) )
	end
	
	
	
	if  ministerCountry:GetTotalIC() < 6 then
		changes:SetAt( CDistributionSetting._LEADERSHIP_RESEARCH_, CFixedPoint( 0 ) )
	else
		changes:SetAt( CDistributionSetting._LEADERSHIP_RESEARCH_, CFixedPoint( 100 ) )
	end


	local factor_left = 100
	for i = 0, table.getn(PRIO_SETTINGS) do -- normalize
		local index = PRIO_SETTINGS[i]
		local factor = changes:GetAt( index ):Get()
		factor = math.min(factor, factor_left)
		factor_left = math.max(factor_left - factor, 0.0)
						
		changes:SetAt( index, CFixedPoint(factor) )
	end
	
	local command = CChangeLeadershipCommand( ministerCountry:GetCountryTag(), changes )
	ai:Post( command )
end


function TechMinister_Tick(minister)
	local ministerCountry = minister:GetCountry()
	BalanceLeadershipSliders( minister:GetOwnerAI(), ministerCountry )
	local i = ministerCountry:GetAllowedResearchSlots() - ministerCountry:GetNumberOfCurrentResearch()
	if i > 0 then

		local techList = ProposeResearch(minister)
		i = math.min(i, table.getn(techList) )

		for j = 0, i do
			local command = CStartResearchCommand( minister:GetCountryTag(), techList[ j+1 ][2] )
			minister:GetOwnerAI():Post( command )
			
		end
	end
end

function EvaluateCurrentResearch(minister)
	-- sort list of research if priorities changed
	
end

function ProposeResearch(minister)
	local sortedTechs = {}
	local ministerCountry = minister:GetCountry()
	local ministerTag = minister:GetCountryTag()

	for tech in CTechnologyDataBase.GetTechnologies() do
		if  minister:CanResearch( tech ) and tech:IsValid() then
			local score = CalculateTechScore( minister, ministerCountry, tech )
			score = score + math.mod( CCurrentGameState.GetAIRand(), 2) -- lets add a small random factor for variety
			table.insert( sortedTechs, {score, tech} )
		end
	end
	table.sort( sortedTechs, function(x, y) return x[1] > y[1] end ) -- highest score first	
	
	--Utils.LUA_DEBUGOUT(tostring(ministerTag) .. ".sortedTechs = {")
	--for _, debugtech in ipairs(sortedTechs) do
	--	if debugtech == nil then
	--		Utils.LUA_DEBUGOUT("nil in list")
	--	else
	--		local techModifiers = minister:GetTechModifers()
	--		Utils.LUA_DEBUGOUT( "(" .. tostring(debugtech[1]) .. ", (" .. tostring(debugtech[2]:GetKey()) .. ") - mod: " .. 100 * techModifiers:GetAt( debugtech[2]:GetIndex() ) )
	--	end
	--
	--end
	--Utils.LUA_DEBUGOUT("}\n")
	
	return sortedTechs
end

function CalculateTechScore( minister, ministerCountry, tech )
	--local logme = false
	local techStatus = ministerCountry:GetTechnologyStatus()
	local score = 0
	
	--if tostring(tech:GetKey()) == "tank_brigade"   then
	--	logme = true
	--end
	
	-- if logme then
	--		Utils.LUA_DEBUGOUT( "CalculateTechScore for " .. tostring(tech:GetKey()) .. "xxxxxx" )
	--end
	
	--Utils.LUA_DEBUGOUT( "CalculateTechScore for " .. tostring(tech:GetKey()) .. "xxxx" )
	
	-- factor in how good we are at research
	local researchCount = 0
	local researchFactor = 0
	for bonus in tech:GetResearchBonus() do
		local ability = ministerCountry:GetAbility( bonus._pCategory )
		researchFactor = researchFactor + ability * bonus._vWeight:Get()
		researchCount = researchCount + 1
	end
	if researchCount > 0 then
		score = researchFactor / researchCount
	else
		score = researchFactor / 10.0
	end
	
	
	
	--if logme then
	--	Utils.LUA_DEBUGOUT( "researchFactor " .. tostring(researchFactor) )
	--	Utils.LUA_DEBUGOUT( "score " .. tostring(score) )
	--end
	
	if score <= 0.001 then
		score = 1.0
	end

	local techLvl = techStatus:GetLevel(tech)
	local techCost = techStatus:GetCost( tech, techLvl + 1 ):Get() / 2.0
	
	--if logme then
	--	Utils.LUA_DEBUGOUT( "techLvl " .. tostring(techLvl) )
	--	Utils.LUA_DEBUGOUT( "techCost " .. tostring(techCost) )
	--end
	
	-- extra year penalty
	local nYear = techStatus:GetYear(tech, techLvl + 1 );
	nYear = nYear - CCurrentGameState.GetCurrentDate():GetYear();
	if nYear > 2 then
		return -1000 -- never research so far ahead
	elseif nYear > 0 then
		techCost = techCost * (1 + 0.1 * nYear)
	end
	
	--if logme then
	--	Utils.LUA_DEBUGOUT( "techCost " .. tostring(techCost) )
	--	Utils.LUA_DEBUGOUT( "score " .. tostring(score) )
	--	Utils.LUA_DEBUGOUT( "score " .. tostring(-techCost	/ score) )
	--end
	
	local score = -techCost	/ score
	
	-- note, score is negative here
	if tech:IsOneLevelOnly() then
		score = score * 0.75
	end
	
	--if logme then
	--	Utils.LUA_DEBUGOUT( "score (level/enable)" .. tostring(score) )
	--end
	
	local techFolder = tech:GetFolder()
	local techFolderName = tostring(tech:GetFolder():GetKey())
	local techModifiers = minister:GetTechModifers()
	local folderModifiers = minister:GetFolderModifers()
	
	if techFolderName == "land_doctrine_folder" then
		local landModifier = folderModifiers:GetAt( CTechnologyDataBase.GetFolderIndex("infantry_folder") )
		landModifier = math.max(landModifier, folderModifiers:GetAt( CTechnologyDataBase.GetFolderIndex("armour_folder") ))
		score = score * 0.5 * (1.0 - landModifier)
	elseif techFolderName == "naval_doctrine_folder" then
		local navalModifier = folderModifiers:GetAt( CTechnologyDataBase.GetFolderIndex("capitalship_folder") )
		navalModifier = math.max(navalModifier, folderModifiers:GetAt( CTechnologyDataBase.GetFolderIndex("smallship_folder") ))
		score = score * 0.5 * (1.0 - navalModifier)
		if ministerCountry:GetNumOfPorts() > 2 then
			score = score * 0.75
		end
	elseif techFolderName == "air_doctrine_folder" then
		local airModifier = folderModifiers:GetAt( CTechnologyDataBase.GetFolderIndex("fighter_folder") )
		airModifier = math.max(airModifier, folderModifiers:GetAt( CTechnologyDataBase.GetFolderIndex("bomber_folder") ))
		score = score * 0.75 * (1.0 - airModifier)
		if ministerCountry:GetNumOfAirfields() > 2 then
			score = score * 0.75
		end
	end
	
	if techFolderName == "theory_folder" then
		score = score * 3 -- not so interested in these unless we got something better
	end
	
	--if logme  then
	--	Utils.LUA_DEBUGOUT(  tostring(tech:GetKey()) .. " - " .. techModifiers:GetAt( tech:GetIndex() ) .. " - score: " .. score)
	--end

	-- enabling a new unit is really good.
	if tech:GetEnableUnit():IsValid() then
		score = score * 0.75
	-- units not in use, dont bother
	elseif techModifiers:GetAt( tech:GetIndex() ) < 0.001
		and (techFolderName == "infantry_folder" 
	 	or techFolderName == "armour_folder"
	 	or techFolderName == "smallship_folder"
	 	or techFolderName == "capitalship_folder"
	 	or techFolderName == "fighter_folder"
	 	or techFolderName == "bomber_folder"
	 	or techFolderName == "bomber_folder")
	then
		score = score - 1000
	end
	
	-- specific techs who need love
	local techName = tostring(tech:GetKey())
	
	if techname == "operational_level_organisation" then -- always useful for ai
		score = score * 0.75
	elseif techname == "industral_production" then
		score = score * 0.75
	elseif techname == "industral_efficiency" then
		score = score * 0.75
	elseif techname == "supply_production" then
		score = score * 0.75
	end
	
	score = score * (1.0 - techModifiers:GetAt( tech:GetIndex() )*2)
	--score = score + 1000 * techModifiers:GetAt( tech:GetIndex() )
	
	
	--if logme then
	--	Utils.LUA_DEBUGOUT("finalscore: " .. tostring(score))
	--	Utils.LUA_DEBUGOUT( "xxxxxxxxxxxxxxxxx\n" )
	--end

	
	return Utils.CallScoredCountryAI(ministerCountry:GetCountryTag(), 'CalculateTechScore', score, ministerCountry, tech )
end

